//
//  Bus.m
//  HP41
//
//  Created by greg on Wed Aug 13 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "Bus.h"
#import "CPU.h"
#import "RomChip.h"
#import "RamChip.h"
#import "Peripheral.h"
#import "DataOperations.h"

@implementation Bus

- (void) dealloc {
    int i;
    for (i = 0; i < 0x10; i++)
        [self->romChips[i] release];
    for (i = 0; i < 0x40; i++)
        [self->ramChips[i] release];
    for (i = 0; i < 0x100; i++)
        [self->peripherals[i] release];
    [super dealloc];
}

- (void) installRomChip: (RomChip *) chip inSlot: (int) slot {
    // Plug ROM chip into the bus at the given location (0x0-0xF),
    // replacing any previous chip at that address.
    [self->romChips[slot] autorelease];
    self->romChips[slot] = [chip retain];
}

- (void) installRamChip: (RamChip *) chip inSlot: (int) slot {
    // Plug RAM chip into the bus at the given location (0x00-0x3F),
    // replacing any previous chip at that address.
    [self->ramChips[slot] autorelease];
    self->ramChips[slot] = [chip retain];
}

- (void) installPeripheral: (id <Peripheral>) periph inSlot: (Bits_8) slot {
    // Plug peripheral into the bus at the given location,
    // replacing any previous peripheral at that address.
    [self->peripherals[slot] pluggedIntoBus: nil];
    [self->peripherals[slot] autorelease];
    self->peripherals[slot] = [periph retain];
    [periph pluggedIntoBus: self];
}

- (BOOL) ramChipInSlot: (int) slot {
    // Test whether there is a RAM chip at the given location (0x00-0x3F).
    return self->ramChips[slot] != nil;
}

- (BOOL) readRomLocation: (int) addr into: (int *) data {
    // Read ROM location at the given address and return true. 
    // If there is no ROM at that address, sets data to 0 and returns false.
    RomChip *rom = self->romChips[addr >> 12];
    if (rom) {
        *data = [rom readLocation: addr & 0xfff];
        return YES;
    }
    else {
        *data = 0;
        return NO;
    }
}

- (BOOL) writeRomLocation: (int) addr from: (int) data {
    RomChip *rom = self->romChips[addr >> 12];
    return [rom writeLocation: addr & 0xfff from: data];
}

- (BOOL) readRamLocation: (int) reg ofChip: (int) slot into: (Digits_14) data {
    // Read specified location of specified chip. If chip or location is
    // nonexistent, set data to 0 and return false.
    RamChip *chip = self->ramChips[slot];
    if (chip)
        return [chip readLocation: reg into: data];
    else {
        FillDigits(data, 0, 14);
        return NO;
    }
}

- (BOOL) writeRamLocation: (int) reg ofChip: (int) slot from: (const Digits_14) data {
    // Write to specified location of specified chip. If chip or location is
    // nonexistent, do nothing and return false.
    RamChip *chip = self->ramChips[slot];
    if (chip)
        return [chip writeLocation: reg from: data];
    else
        return NO;
}

- (void) readFromRegister: (Bits_4) reg ofPeripheral: (Bits_8) slot into: (Digits_14) data {
    id <Peripheral> periph = self->peripherals[slot];
    if (periph)
        [periph readFromRegister: reg into: data];
    else
         FillDigits(data, 0, 14);
}

- (void) writeToRegister: (Bits_4) reg ofPeripheral: (Bits_8) slot from: (Digits_14) data {
    id <Peripheral> periph = self->peripherals[slot];
    [periph writeToRegister: reg from: data];
}

- (void) writeDataToPeripheral: (Bits_8) slot from: (Digits_14) data {
    id <Peripheral> periph = self->peripherals[slot];
    [periph writeDataFrom: data];
}

- (void) abortInstruction: (NSString *) message {
    [self->cpu abortInstruction: message];
}

@end
