#import <stdio.h>
#import "ShapedWindow.h"
#import "KeyGroup.h"
#import "Key.h"

@interface Key(private)

- (void) notifyKeyGroup;

@end

@implementation Key

//- (id) initWithFrame: (NSRect) frameRect
//{
//    [super initWithFrame:frameRect];
//    self->pressed = NO;
//    return self;
//}

- (int) keyCode {
    return self->keyCode;
}

- (void) setKeyCode: (int) code {
    self->keyCode = code;
}

// Change key state and redisplay.

- (void) setPressed: (BOOL) state {
    self->pressed = state;
    [self setNeedsDisplay: YES];
}

// Provide feedback for when activated by a Macintosh key event.

- (void) showFeedback: (BOOL) down forCode: (int) code {
    [self setPressed: down];
}

// Each Key is a subview of its corresponding KeyGroup.

- (void) awakeFromNib {
    self->keyGroup = (KeyGroup *) [self superview];
    [self->keyGroup registerKey: self];
}

// To draw a pressed-down key, we grab the relevant part of the
// calculator image (obtained from the backgroundView of the
// containing ShapedWindow), draw it a couple of pixels lower
// down, and fill the space above it with black. For a key that's
// not pressed, we don't draw anything, and let the background
// image show through.

- (void) drawRect: (NSRect) rect
{
    if (self->pressed) {
        id window = [self window];
        NSImageView *bgView = [window backgroundView];
        NSImage *calcImage = [bgView image];
        NSRect keyRect = [self convertRect: [self bounds] toView: bgView];
        int d = 2;
        NSRect srcRect = NSMakeRect(
            keyRect.origin.x, keyRect.origin.y + d,
            keyRect.size.width, keyRect.size.height - d);
        NSRect fillRect = NSMakeRect(
            0, srcRect.size.height,
            keyRect.size.width, d);
        NSPoint p = NSMakePoint(0, 0);
        [calcImage compositeToPoint: p fromRect: srcRect 
            operation: NSCompositeCopy];
        [[NSColor blackColor] set];
        NSRectFill(fillRect);
    }
}

// When the mouse is pressed or released, we send a message to
// the containing KeyGroup.
//
// We ignore mouse events with the Control key pressed. This
// makes it possible to generate isolated key presses and
// releases for debugging purposes.

- (void) mouseDown: (NSEvent *) e {
    if (!([e modifierFlags] & NSControlKeyMask)) {
        //self->pressed = YES;
        //[self setNeedsDisplay: YES];
        [self setPressed: YES];
        [self notifyKeyGroup];
    }
}

- (void) mouseUp: (NSEvent *) e {
    if (!([e modifierFlags] & NSControlKeyMask)) {
        //self->pressed = NO;
        //[self setNeedsDisplay: YES];
        [self setPressed: NO];
        [self notifyKeyGroup];
    }
}

- (void) notifyKeyGroup {
    [self->keyGroup key: self pressed: self->pressed];
}

// We need to intercept mouseDragged events too, otherwise they
// pass through to the ShapedWindow and it starts getting dragged
// around the screen.

- (void) mouseDragged: (NSEvent *) e {
}

- (BOOL) acceptsFirstMouse: (NSEvent *) event {
    return YES;
}

@end
