//
//  RegisterDisplay.m
//  HP41
//
//  Created by greg on Wed Aug 13 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import <ApplicationServices/ApplicationServices.h>
#import "RegisterDisplay.h"
#import "DebugController.h"

@implementation RegisterDisplay

- (int) numColumns {
    return 0;
}

- (BOOL) acceptsFirstResponder {
    return YES;
}

- (BOOL) becomeFirstResponder {
    self->cellIsSelected = YES;
    self->selectedLine = 0;
    self->selectedColumn = [self numColumns] - 1;
    [self setNeedsDisplay: YES];
    return YES;
}

- (BOOL) resignFirstResponder {
    if (self->cellIsSelected) {
        self->cellIsSelected = NO;
        [self setNeedsDisplay: YES];
    }
    return YES;
}

- (void) mouseDown: (NSEvent *) event {
    NSPoint p = [self convertPoint: [event locationInWindow] fromView: nil];
    NSRect bounds = [self bounds];
    self->selectedLine = (bounds.size.height - p.y) / (bounds.size.height / [self numLines]);
    self->selectedColumn = (bounds.size.width - p.x) / (bounds.size.width / [self numColumns]);
    self->cellIsSelected = YES;
    [self setNeedsDisplay: YES];
}

- (void) keyDown: (NSEvent *) event {
    if (self->cellIsSelected) {
        NSString *s = [[event characters] uppercaseString];
        unichar c = [s characterAtIndex: 0];
        //printf("RegisterDisplay keyDown: 0x%04x\n", c);
        switch (c) {
            case 0xd:
            case 0x3:
                [self resignFirstResponder];
                break;
            case 0x9:
                [super keyDown: event];
                break;
            case 0xf700:
                [self moveSelectionLeft: 0 down: -1];
                break;
            case 0xf701:
                [self moveSelectionLeft: 0 down: 1];
                break;
            case 0xf702:
                [self moveSelectionLeft: 1 down: 0];
                break;
            case 0xf703:
                [self moveSelectionLeft: -1 down: 0];
                break;
            default:
                if ((c >= '0' && c <= '9') || (c >= 'A' && c <= 'F')) {
                    int d = c - '0';
                    if (d > 9)
                        d -= 7;
                    [self setDigit: self->selectedColumn ofLine: self->selectedLine to: d];
                    [self moveSelectionLeft: -1 down: 0];
                    [self->debugController registerChanged: self];
                }
        }
    }
}

- (void) moveSelectionLeft: (int) dh down: (int) dv {
    if (self->cellIsSelected) {
        int numLines = [self numLines];
        int numColumns = [self numColumns];
        self->selectedColumn = (self->selectedColumn + dh + numColumns) % numColumns;
        self->selectedLine = (self->selectedLine + dv + numLines) % numLines;
        [self setNeedsDisplay: YES];
    }
}

- (void) drawRect: (NSRect) rect {
    [self drawBackground];
    [self drawData];
}

- (void) drawBackground {
    [[NSColor blackColor] set];
    NSRectFill([self bounds]);
}

- (void) drawData {
    NSRect bounds;
    int i, d, numLines, numDigits, line, lineHeight;
    float x, y, digitCellWidth, digitCharWidth, digitCharHeight, digitLeftMargin, digitBaseLine;
    Digits_14 buffer;
    
    bounds = [self bounds];
    numLines = [self numLines];
    numDigits = [self numColumns];
    lineHeight = bounds.size.height / numLines;
    digitCellWidth = bounds.size.width / numDigits;
    digitCharWidth = self->digitSize.width;
    digitCharHeight = self->digitSize.height;
    digitLeftMargin = (digitCellWidth - digitCharWidth) / 2;
    digitBaseLine = (lineHeight - digitCharHeight) / 2;
    for (line = 0; line < numLines; line++) {
        if ([self getDigitsForLine: line into: buffer]) {
        #if 0
            NSString *s;
            for (i = 0; i < numDigits; i++) {
                d = [self getDigit: i ofLine: line];
                s = [NSString stringWithFormat: @"%X", d];
                x = bounds.size.width - (i + 1) * digitCellWidth + digitLeftMargin;
                y = bounds.size.height - (line + 1) * lineHeight + digitBaseLine;
                [s drawAtPoint: NSMakePoint(x, y) withAttributes: self->attributes];
            }
        #elif 0
            NSString *s;
            char buffer[29], *bufp;
            static char *hex = "0123456789ABCDEF";
            bufp = buffer;
            for (i = numDigits - 1; i >= 0; i--) {
                d = [self getDigit: i ofLine: line];
                *bufp++ = hex[d];
                *bufp++ = ' ';
            }
            *bufp = 0;
            s = [NSString stringWithCString: buffer];
            x = digitLeftMargin;
            y = bounds.size.height - (line + 1) * lineHeight + digitBaseLine;
            [s drawAtPoint: NSMakePoint(x, y) withAttributes: self->attributes];
        #elif 1
            const char *fontName = [[self->font fontName] UTF8String];
            int fontSize = [self->font pointSize];
            //[[self->attributes objectForKey: NSForegroundColorAttributeName] set];
            //[self->foregroundColor set];
            CGContextRef ctx = [[NSGraphicsContext currentContext] graphicsPort];
            CGContextSelectFont(ctx, fontName, fontSize, kCGEncodingMacRoman);
            CGContextSetTextDrawingMode(ctx, kCGTextFill);
            CGContextSetTextMatrix(ctx, CGAffineTransformMakeTranslation(0, 0));
            for (i = 0; i < numDigits; i++) {
                static char *hex = "0123456789ABCDEF";
                x = bounds.size.width - (i + 1) * digitCellWidth;
                y = bounds.size.height - (line + 1) * lineHeight;
                //d = [self getDigit: i ofLine: line];
                d = buffer[i];
                if (d >= 0 && d <= 0xf)
                    [self->foregroundColor set];
                else
                    [[NSColor redColor] set];
                d &= 0xf;
                if (self->cellIsSelected && line == self->selectedLine && i == self->selectedColumn) {
                    CGContextFillRect(ctx, CGRectMake(x, y, digitCellWidth, lineHeight));
                    [self->backgroundColor set];
                }
                CGContextShowTextAtPoint(ctx, x + digitLeftMargin, y + digitBaseLine, &hex[d], 1);
            }
        #endif
        }
    }
}

- (int) numLines {
    return 1;
}

//- (Digit) getDigit: (int) index ofLine: (int) line {
//    return 0;
//}

- (BOOL) getDigitsForLine: (int) line into: (Digits_14) data {
    return NO;
}

- (void) setDigit: (int) index ofLine: (int) line to: (int) value {
}

@end
