#import "ShapedWindow.h"

@implementation ShapedWindow

- (id) initWithContentRect: (NSRect) contentRect 
        styleMask: (unsigned int) style 
        backing: (NSBackingStoreType) bufferingType 
        defer: (BOOL) defer 
{
    NSWindow* result = 
        [super initWithContentRect: contentRect 
            styleMask: NSBorderlessWindowMask 
            backing: NSBackingStoreBuffered 
            defer: defer];
    [result setBackgroundColor: [NSColor clearColor]];
    [result setOpaque: NO];
    [result setHasShadow: YES];
    return result;
}

- (BOOL) canBecomeMainWindow {
    return YES;
}

- (BOOL) canBecomeKeyWindow
{
    return YES;
}

- (void) mouseDown: (NSEvent *) theEvent
{    
    NSRect  windowFrame = [self frame];

    initialLocation = [self convertBaseToScreen:[theEvent locationInWindow]];
    initialLocation.x -= windowFrame.origin.x;
    initialLocation.y -= windowFrame.origin.y;
}

- (void) mouseDragged: (NSEvent *) theEvent
{
    NSPoint currentLocation;
    NSPoint newOrigin;
    NSRect  screenFrame = [[NSScreen mainScreen] frame];
    NSRect  windowFrame = [self frame];

    currentLocation = [self convertBaseToScreen:[self mouseLocationOutsideOfEventStream]];
    newOrigin.x = currentLocation.x - initialLocation.x;
    newOrigin.y = currentLocation.y - initialLocation.y;
    
    // Don't let window get dragged up under the menu bar
    if( (newOrigin.y+windowFrame.size.height) > (screenFrame.origin.y+screenFrame.size.height) ){
        newOrigin.y=screenFrame.origin.y + (screenFrame.size.height-windowFrame.size.height);
    }
    
    //go ahead and move the window to the new location
    [self setFrameOrigin:newOrigin];
}

- (NSImageView *) backgroundView {
    return backgroundImage;
}

- (BOOL) validateMenuItem: (id) item {
    SEL action = [item action];
    if (action == @selector(makeKeyAndOrderFront:)) {
        [item setState: [self isVisible]];
        return YES;
    }
    return [super validateMenuItem: item];
}

@end
