//
//  SoundOutput.m
//  X-41
//
//  Created by greg on Sat Aug 30 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import <Carbon/Carbon.h>
#import "SoundOutput.h"

#define DEBUG 0

#define DBPRINTF if (DEBUG) printf

//--------------------------------------------------------------------------------

// Sample rate is equal to the CPU word execution rate.
#define SAMPLE_RATE (1.0 / 155e-6)

// SOUND_LATENCY is the delay between the CPU producing sound
// samples and sending those samples to the Sound Manager, to
// smooth out fluctuations in instruction execution rate by
// the emulator.
#define SOUND_LATENCY 0.1

// TOTAL_BUFFERS is the total number of sound buffers to allocate.
// LATENCY_BUFFERS is the number of buffers that will be filled during
// the latency period.
// SILENCE_BUFFERS is the number of buffers after which sound sample
// processing will be stopped if there is no sound activity.
#define TOTAL_BUFFERS 4 // 8
#define LATENCY_BUFFERS 2
#define SILENCE_BUFFERS 2

// Amplitute of the generated square wave
#define AMPLITUDE 10

//--------------------------------------------------------------------------------

// Fixed point representation of the sample rate.
#define SAMPLE_RATE_FIXED ((unsigned int)(65536.0 * SAMPLE_RATE))

// We will wait until at least LATENCY_BUFFERS buffers of sound are filled before
// starting to send them to the sound channel. The buffer size is
// calculated so that LATENCY_BUFFERS buffers equals the specified latency.
#define SAMPLES_PER_BUFFER ((unsigned int)(SOUND_LATENCY * SAMPLE_RATE / LATENCY_BUFFERS))

// Sample values for high and low levels of the square wave
#define LOW_SAMPLE (0x80 - AMPLITUDE)
#define HIGH_SAMPLE (0x80 + AMPLITUDE)

typedef unsigned char SoundSample;

typedef struct SoundBuffer {
    struct SoundBuffer *next;
    BOOL inUse;
    SoundHeader header;
    SoundSample samples[SAMPLES_PER_BUFFER];
} SoundBuffer;

static SndChannelPtr gSndChannel;
static SoundBuffer gBuffers[TOTAL_BUFFERS];
static int gBufferSendPtr;
static int gBufferAllocPtr;
static int gNumPendingBuffers;
static SoundBuffer *gCurrentBuffer;
static BOOL gBufferingSound;
static int gConstantSampleCount;
static SoundSample gLastSample = LOW_SAMPLE;

/* static pascal void SoundCallback(SndChannelPtr chan, SndCommand *cmd) {
    //DBPRINTF("Buffer %d is empty\n", cmd.param1);
    gBuffers[cmd->param1].inUse = NO;
}
 */
static void StartUpSoundChannel(void) {
    if (!gSndChannel) {
        DBPRINTF("Allocating sound channel\n");
        //SndNewChannel(&gSndChannel, sampledSynth, 0, (SndCallBackUPP)SoundCallback);
    }
}

static void ShutDownSoundChannel(void) {
    DBPRINTF("Deallocating sound channel\n");
    //SndDisposeChannel(gSndChannel, YES);
    gSndChannel = NULL;
}

static void IncBufferPtr(int *ptr) {
    if (++*ptr == TOTAL_BUFFERS)
        *ptr = 0;
}

static void InitBuffer(SoundBuffer *buf) {
    buf->header.sampleRate = SAMPLE_RATE_FIXED;
    buf->header.samplePtr = (Ptr) buf->samples;
    buf->header.length = 0;
}

static int gDroppedSampleCount;

static SoundBuffer *GetBuffer(void) {
    SoundBuffer *buf = gCurrentBuffer;
    if (!buf) {
        buf = &gBuffers[gBufferAllocPtr];
        if (buf->inUse) {
            if (gDroppedSampleCount == 0)
                DBPRINTF("Buffer %d is in use\n", gBufferAllocPtr);
            ++gDroppedSampleCount;
            return NULL;
        }
        if (gDroppedSampleCount) {
            DBPRINTF("Buffer %d is free after %d dropped samples\n", gBufferAllocPtr, gDroppedSampleCount);
            gDroppedSampleCount = 0;
        }
        DBPRINTF("Starting to fill buffer %d\n", gBufferAllocPtr);
        InitBuffer(buf);
        buf->inUse = YES;
        IncBufferPtr(&gBufferAllocPtr);
        gCurrentBuffer = buf;
    }
    return buf;
}

static void FinishBuffer(void) {
    if (gCurrentBuffer) {
        DBPRINTF("Finished filling buffer %d\n", gCurrentBuffer - gBuffers);
        ++gNumPendingBuffers;
        gCurrentBuffer = NULL;
    }
}

static void SendPendingBuffers(void) {
    while (gNumPendingBuffers) {
        if (!gSndChannel)
            StartUpSoundChannel();
        DBPRINTF("Sending buffer %d\n", gBufferSendPtr);
        SoundBuffer *buf = &gBuffers[gBufferSendPtr];
        SndCommand cmd;
        cmd.cmd = bufferCmd;
        cmd.param1 = 0;
        cmd.param2 = (long)&buf->header;
        //result = SndDoCommand(gSndChannel, &cmd, YES);
        cmd.cmd = callBackCmd;
        cmd.param1 = gBufferSendPtr;
        cmd.param2 = 0;
       // result = SndDoCommand(gSndChannel, &cmd, YES);
        IncBufferPtr(&gBufferSendPtr);
        --gNumPendingBuffers;
    }
}

int SoundAvailableBufferSpace(void) {
    int p = gBufferAllocPtr;
    int numFreeBuffers = 0;
    while (numFreeBuffers < TOTAL_BUFFERS && !gBuffers[p].inUse) {
        ++numFreeBuffers;
        IncBufferPtr(&p);
    }
    int numSamples = numFreeBuffers * SAMPLES_PER_BUFFER;
    DBPRINTF("Room for %d samples\n", numSamples);
    return numSamples;
}

void SoundOutputForWordTime(int state) {
    SoundSample sample = state ? HIGH_SAMPLE : LOW_SAMPLE;
    if (!gBufferingSound) {
        if (sample == gLastSample)
            return;
        DBPRINTF("Starting to buffer sound\n");
        gBufferingSound = YES;
    }
    SoundBuffer *buf = GetBuffer();
    if (buf) {
        buf->samples[buf->header.length++] = sample;
        if (buf->header.length == SAMPLES_PER_BUFFER) {
            FinishBuffer();
            if (gSndChannel || gNumPendingBuffers >= LATENCY_BUFFERS)
                SendPendingBuffers();
        }
    }
    if (sample == gLastSample) {
        if (++gConstantSampleCount >= (SILENCE_BUFFERS * SAMPLES_PER_BUFFER))
            FlushAndSuspendSoundOutput();
    }
    else
        gConstantSampleCount = 0;
    gLastSample = sample;
}

void FlushAndSuspendSoundOutput(void) {
    DBPRINTF("Stopping buffering sound\n");
    FinishBuffer();
    SendPendingBuffers();
    gBufferingSound = NO;
}

static BOOL AnyBuffersInUse(void) {
    int i;
    for (i = 0; i < TOTAL_BUFFERS; i++)
        if (gBuffers[i].inUse)
            return YES;
    return NO;
}

void SoundTimeslice(void) {
    if (!gBufferingSound && gSndChannel && !AnyBuffersInUse())
        ShutDownSoundChannel();
}


