//
//  DataOperations.m
//  HP41
//
//  Created by greg on Thu Aug 14 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "DataOperations.h"

void CopyDigits(const Digit *src, Digit *dst, int count) {
   while (count--)
        *dst++ = *src++;
}

void FillDigits(Digit *dst, Digit value, int count) {
   while (count--)
        *dst++ = value;
}

void BitsToDigits(int bits, Digit *digits, int ndigits) {
   while (ndigits--) {
        *digits++ = bits & 0xf;
        bits >>= 4;
    }
}

void DigitsToBits(Digit *digits, int *bits, int nbits) {
    int result = 0;
    int ndigits = (nbits + 3) >> 2;
    Digit *dp = digits + ndigits;
    while (ndigits--)
        result = result << 4 | *--dp;
    *bits = result & ((1 << nbits) - 1);
}

void DigitsToBitsWrap(Digits_14 digits, int *bits, int start, int count) {
    int result = 0;
    int i = start;
    int j = 0;
    while (count--) {
        result |= digits[i] << j;
        ++i;
        if (i > 13)
            i = 0;
        j += 4;
    }
    *bits = result;
}

void BitsToDigitsWrap(int bits, Digits_14 digits, int start, int count) {
    int i = start;
    while (count--) {
        digits[i] = bits & 0xF;
        bits >>= 4;
        ++i;
        if (i > 13)
            i = 0;
    }
}

void AddOrSubtractDigits(
    ArithOp op, ArithMode mode,
    Digit src1[], Digit src2[], Digit dst[],  
    int start, int count, 
    Bit *carry, Bit *zero)
{
    // Perform a hex or decimal add or subtract operation on the specified
    // range of digits. Carry is an inout parameter specifying the initial
    // carry and returning the final carry. Zero is an out parameter returning
    // true if all digits of the result are zero.
    int d, m;
    int c = *carry;
    int z = 1;
    int i = start;
    if (op == SUB)
        m = (mode == DEC_MODE) ? 0x9 : 0xF;
    while (count--) {
        if (op == ADD)
            d = src1[i] + src2[i] + c;
        else
            d = src1[i] + m - src2[i] + c;
        if (mode == DEC_MODE && d > 9)
            d += 6;
        c = d >> 4;
        d &= 0xf;
        if (d)
            z = 0;
        dst[i] = d;
        ++i;
    }
    *carry = c;
    *zero = z;
}

void ExchangeDigits(Digit x[], Digit y[], int start, int count) {
    int i, d;
    i = start;
    while (count--) {
        d = x[i];
        x[i] = y[i];
        y[i] = d;
        ++i;
    }
}

void ShiftDigitsLeft(Digit x[], int start, int count) {
    int i;
    for (i = start + count - 1; i > start; i--)
        x[i] = x[i-1];
    x[start] = 0;
}

void ShiftDigitsRight(Digit x[], int start, int count) {
    int i;
    int end = start + count;
    for (i = start + 1; i < end; i++)
        x[i-1] = x[i];
    x[end-1] = 0;
}

void AndDigits(Digit x[], Digit y[], Digit z[], int start, int count) {
    int i;
    for (i = start; i < count; i++)
        z[i] = x[i] & y[i];
}

void OrDigits(Digit x[], Digit y[], Digit z[], int start, int count) {
    int i;
    for (i = start; i < count; i++)
        z[i] = x[i] | y[i];
}

